# Parliament Data API Wrapper (Ruby)
[parliament-ruby][parliament-ruby] is a gem created by the [Parliamentary Digital Service][pds] to allow easy communication with the internal parliament data API.

[![Gem][shield-gem]][info-gem] [![Build Status][shield-travis]][info-travis] [![Test Coverage][shield-coveralls]][info-coveralls] [![License][shield-license]][info-license]

> **NOTE:** This gem is in active development and is likely to change at short notice. It is not recommended that you use this in any production environment.

### Contents
<!-- START doctoc generated TOC please keep comment here to allow auto update -->
<!-- DON'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->


- [Requirements](#requirements)
- [Installation](#installation)
- [Usage](#usage)
  - [Setting up a connection](#setting-up-a-connection)
    - [Setting a base URL 'globally'](#setting-a-base-url-globally)
  - [Building an 'endpoint'](#building-an-endpoint)
  - [Methods](#methods)
    - [`#get`](#get)
    - [`#filter`](#filter)
    - [`#sort_by`](#sort_by)
    - [`#reverse_sort_by`](#reverse_sort_by)
  - [`Parliament::Utils`](#parliamentutils)
    - [Methods](#methods-1)
      - [`Parliament::Utils.sort_by`](#parliamentutilssort_by)
      - [`Parliament::Utils.reverse_sort_by`](#parliamentutilsreverse_sort_by)
- [Getting Started with Development](#getting-started-with-development)
  - [Running the tests](#running-the-tests)
- [Contributing](#contributing)
- [License](#license)

<!-- END doctoc generated TOC please keep comment here to allow auto update -->


## Requirements
[parliament-ruby][parliament-ruby] requires the following:
* [Ruby][ruby] - [click here][ruby-version] for the exact version
* [Bundler][bundler]


## Installation
```bash
gem 'parliament-ruby'
```


## Usage
This gem's main function is fetching an n-triples from a remote server and converting it into linked ruby objects.

> **Note:** Comprehensive class documentation can be found on [rubydocs][rubydocs].

### Setting up a connection
In order to connect to a remote server, we first need to set a base_url value, from which we will build an 'endpoint'. The base_url should be the beginning of a url without the trailing slash. For example `http://example.com` instead of `http://example.com/`.
```ruby
parliament = Parliament::Request.new(base_url: 'http://test.com')
```

#### Setting a base URL 'globally'
Within code you can set a global base URL using the following snippet.
```ruby
Parliament::Request.base_url = 'http://test.com'

# The base_url should be set for all new objects
Parliament::Request.new.base_url #=> 'http://test.com'

# You can still override the base_url on an instance by instance basis
Parliament::Request.new(base_url: 'http://example.com').base_url #=> 'http://example.com'
```

Alternatively, you can set the environment variable `PARLIAMENT_BASE_URL` on your machine and we will automatically use that.
```ruby
ENV['PARLIAMENT_BASE_URL'] #=> 'http://example.com'

Parliament::Request.base_url #=> nil

Parliament::Request.new.base_url #=> 'http://example.com'
```


### Building an 'endpoint'
Now that we have a `base_url` set, we will want to build an 'endpoint' such as: `http://test.com/parties/current`.

An endpoint is effectively just a full URL to an n-triple file on a remote server.

Building an endpoint is simple, once you have a Parliament::Request object, you do something like this:
```ruby
parliament = Parliament::Request.new(base_url: 'http://test.com') #=>  #<Parliament::Request [...]>

# Target endpoint: 'http://test.com/parties/current'
parliament.parties.current

# Target endpoint: 'http://test.com/parties/123/people/current'
parliament.parties('123').people.current

# Target endpoint: 'http://test.com/people/123/letters/456'
parliament.people('123').letters('456')
```

### Setting headers

#### Setting headers 'globally'
Within the code you can set global headers using the following snippet.
```
Parliament::Request.headers = { 'Accept' => 'Test' }

# The headers should be set for all new objects
Parliament::Request.new.headers #=> { 'Accept' => 'Test' }

# You can still override the headers on an instance by instance basis
Parliament::Request.new(headers: { 'Accept' => 'Test2' }).headers #=> { 'Accept' => 'Test2' }
```

## Getting Started with Development
To clone the repository and set up the dependencies, run the following:
```bash
git clone https://github.com/ukparliament/parliament-ruby.git
cd parliament-ruby
bundle install
```

### Running the tests
We use [RSpec][rspec] as our testing framework and tests can be run using:
```bash
bundle exec rake
```


## Contributing
If you wish to submit a bug fix or feature, you can create a pull request and it will be merged pending a code review.

1. Fork the repository
1. Create your feature branch (`git checkout -b my-new-feature`)
1. Commit your changes (`git commit -am 'Add some feature'`)
1. Push to the branch (`git push origin my-new-feature`)
1. Ensure your changes are tested using [Rspec][rspec]
1. Create a new Pull Request


## License
[parliament-ruby][parliament-ruby] is licensed under the [Open Parliament Licence][info-license].

[ruby]:            https://www.ruby-lang.org/en/
[bundler]:         http://bundler.io
[rspec]:           http://rspec.info
[parliament-ruby]: https://github.com/ukparliament/parliament-ruby
[pds]:             https://www.parliament.uk/mps-lords-and-offices/offices/bicameral/parliamentary-digital-service/
[ruby-version]:    https://github.com/ukparliament/parliament-ruby/blob/master/.ruby-version
[grom]:            https://github.com/ukparliament/grom
[rubydocs]:        http://www.rubydoc.info/github/ukparliament/parliament-ruby/master/file/README.md

[info-gem]:  https://rubygems.org/gems/parliament-ruby
[shield-gem]: https://img.shields.io/gem/v/parliament-ruby.svg

[info-travis]:   https://travis-ci.org/ukparliament/parliament-ruby
[shield-travis]: https://img.shields.io/travis/ukparliament/parliament-ruby.svg

[info-coveralls]:   https://coveralls.io/github/ukparliament/parliament-ruby
[shield-coveralls]: https://img.shields.io/coveralls/ukparliament/parliament-ruby.svg

[info-license]:   http://www.parliament.uk/site-information/copyright/open-parliament-licence/
[shield-license]: https://img.shields.io/badge/license-Open%20Parliament%20Licence-blue.svg
